package AFDX;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
public class Tabu {
	public static int FPNFP(List<Integer> hp, List<Integer> ep, List<Integer> lp, int[] pSize, double[] period, double[] jitter, int i, double R) {
		int res = 0;
		boolean flag = true;
		List<Integer> l = new ArrayList<>();
		int k = hp.size();
		for(int j=0;j<k;j++) {
			l.add(1);
		}
		double J = jitter[i];
		double ipJ = 0;
		int mSize = 0;
		for(int j=0;j<k;j++) {
			res = res+pSize[hp.get(j)];
			J=Math.min(J,jitter[hp.get(j)]);
		}
		for(int j=0;j<lp.size();j++) {
			if(pSize[lp.get(j)] > mSize) {
				mSize = pSize[lp.get(j)];
				ipJ = jitter[lp.get(j)];
				
			}
		}
		res = res+mSize;
		for(int j=0;j<ep.size();j++) {
			res = res+pSize[ep.get(j)];
			J=Math.min(J,jitter[ep.get(j)]);
		}
		J = Math.max(J, ipJ);
		//System.out.println("res: "+i+": "+res);
		while(flag) {
			flag = false;
			int j = 0;
			
			while(j<k) {
				if((double)res*8.0/R+J > period[hp.get(j)]*(l.get(j))){
					flag = true;
					l.set(j,l.get(j)+1);
					res = res+pSize[hp.get(j)];
				}
				j = j+1;
			}
			
		}
		return res;		
	}
	public static int FPFP(List<Integer> hp, List<Integer> ep, List<Integer> lp, int[] pSize, double[] period, double[] jitter, int i, double R) {
		int res = pSize[i];
		boolean flag = true;
		List<Integer> l = new ArrayList<>();
		int k = hp.size();
		for(int j=0;j<k;j++) {
			l.add(1);
		}
		double J = jitter[i];
		for(int j=0;j<k;j++) {
			res = res+pSize[hp.get(j)];
			J=Math.min(J,jitter[hp.get(j)]);
		}
		for(int j=0;j<ep.size();j++) {
			res = res+pSize[ep.get(j)];
			J=Math.min(J,jitter[ep.get(j)]);
		}
		//System.out.println("res: "+i+": "+res);
		while(flag) {
			flag = false;
			int j = 0;		
			while(j<k) {
				if((double)res*8.0/R+J > period[hp.get(j)]*(l.get(j))){
					flag = true;
					l.set(j,l.get(j)+1);
					res = res+pSize[hp.get(j)];
				}
				j = j+1;
			}		
		}
		return res-pSize[i];		
	}
	public static void findSets(int[] priority, List<Integer> hp, List<Integer> ep, List<Integer> lp, int i) {
		int m = priority.length;
		for(int j=0;j<m;j++) {
			if(j==i) continue;
			if(priority[j]<priority[i]) {
				lp.add(j);
			}else if (priority[j] == priority[i]) {
				ep.add(j);
			}else {
				hp.add(j);
			}
		}
	}
	public static void generateDiversity(int[] priority, int numP) {
		int m = priority.length;
		for(int i=0;i<m;i++) {
			Random rand = new Random();		
			int temp = rand.nextInt(numP);
			while(temp == priority[i]) {
				temp = rand.nextInt(numP);
			}
			priority[i] = temp;
		}
	}
	public static void generateNeighbors(List<int[]> neighbors, int n, int[] priority, int numP){
	    int m = priority.length;
	    Random rand = new Random();
	    
		while(neighbors.size()<n) {
			int temp = rand.nextInt(m);
			int[] tempP = priority.clone();
			tempP[temp] = rand.nextInt(numP);
			if(neighbors.contains(tempP)) continue;
			neighbors.add(tempP);
		}
		
	}
	public static double penalty(double[] delay, double[] delayR, double c) {
		double res = 0;
		for(int i=0;i<delay.length;i++) {
			if(delay[i]>delayR[i]) {
				res = c*(delay[i]-delayR[i]);
			}
		}
		return res;
	}
	

	public static double Tabu(int[] priority, int stop, int n, int numP, int[] pSize, double[] period, double[] delayR, double[] jitter, double R, double T, double c, int k, int omega, int flag) {
		int[] current = priority.clone();
		int[] best = priority.clone();
		double bestP = getPenalty(best, pSize, period, delayR, jitter, R, T, c, flag);

		int iter = 0;
		List<int[]> tabu = new ArrayList<>();
	    List<Integer> tabuIter = new ArrayList<>();
	    int stagnation = 0;
		while(iter<stop) {
			iter++;
			if(tabu.contains(current)) {
				generateDiversity(current, numP);
			}
			tabu.add(current);
			tabuIter.add(k);
			List<int[]> neighbors = new ArrayList<>();
			generateNeighbors(neighbors, n, current, numP);
			
			for(int i=0;i<n;i++) {
				current = neighbors.get(i);
				if(tabu.contains(current)) continue;
				double p = getPenalty(current, pSize, period, delayR, jitter, R, T, c, flag);
				if(p<bestP) {
					best = current;
					bestP = p;
					stagnation = 0;
				}else {
					stagnation++;
				}
				tabu.add(current);
				tabuIter.add(k);
				if(stagnation == omega) {
					break;
				}else {
					for(int j=0;i<tabu.size();i++) {
						tabuIter.set(j, tabuIter.get(j)-1);
					}
					for(int j=0;i<tabu.size();i++) {
						if(tabuIter.get(j) <= 0) {
							tabu.remove(j);
							tabuIter.remove(j);
						}
						j--;
					}
				}
			}
			generateDiversity(current, numP);
		}
		//System.out.println("res of Tabu: "+Arrays.toString(best));
		priority = best;
		return bestP;
	}
	public static void writeArrayToExcel(double[][] data, String string) {
        int rowNum = data.length;
        int columnNum = data[0].length;
        try {
            FileWriter fw = new FileWriter(string);
            for (int i = 0; i < rowNum; i++) {
                for (int j = 0; j < columnNum; j++)
                    fw.write(data[i][j]+ "\t"); // tab 
                fw.write("\n"); // 
            }
            fw.close();
        }
        catch (IOException e){
            e.printStackTrace();
        }
	}
	public static double getPenalty(int[] priority, int[] pSize, double[] period, double[] delayR, double[] jitter, double R, double T, double c, int flag) {
		int m = priority.length;
		double[] delay = new double[m];
		for(int i=0;i<m;i++) {
			List<Integer> hp = new ArrayList<>();
			List<Integer> ep = new ArrayList<>();
			List<Integer> lp = new ArrayList<>();
			findSets(priority,hp,ep,lp,i);	
			int res = 0;
			if(flag == 1) {
				res = FPFP(hp, ep, lp, pSize, period, jitter, i, R);
			}else {
				res = FPNFP(hp, ep, lp, pSize, period, jitter, i, R);
			}
				
			//System.out.println(res);
			delay[i] = (R*T+(double)res*8+(double)pSize[i]*8)/R;
			//System.out.println(delay[i]);
		}
		double res = penalty(delay, delayR, c);
		return res;		
	}
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		int numVL = 20;
		double[][] penalty = new double[9][10];
		double[][] penalty1 = new double[9][10];
		List<int[]> priorityRecord = new ArrayList<>();
		for(int numP = 2;numP<=8;numP =numP+6) {
			for(numVL = 20;numVL<=100;numVL=numVL+10) {
				int[] priority = new int[numVL];
				double[] BAG = {2, 4, 8, 16, 32, 64, 128};
				int[] size = {32, 64, 128, 256, 512, 1024, 1500};
				for(int i=0;i<numVL;i++) {
		        	Random rand= new Random();
		        	int temp = rand.nextInt(numP);
		        	priority[i] = temp;
		        }
				int[] previousPriority = new int[numVL-10];
				if(priorityRecord.size()!=0) {
					previousPriority = priorityRecord.get(priorityRecord.size()-1);
				}
				for(int i=0;i<numVL-10;i++) {
					priority[i] = previousPriority[i];
				}
		        
				double[] period = new double[numVL];
				double[] jitter =  new double[numVL];
				for(int i=0;i<priority.length;i++) {
		        	period[i] = BAG[i%7]/1000;
		        	
				}
				int[] pSize = new int[numVL];
				for(int i=0;i<numVL;i++) {
					pSize[i] = size[i%7];
				}
				double[] delayR = new double[numVL];
				for(int i=0;i<numVL;i++) {
					delayR[i] = (double)pSize[i]/64/1000;
				}
				int stop = 5000;
				double R = 500000*40;
				double T = 0.001;
				double c = 100;
				int k = 10;
				int n = 10;
				int omega = 4;
				int flag = 1;
				for(int i=0;i<10;i++) {
					double res = Tabu(priority, stop, n, numP, pSize, period, delayR, jitter,  R,  T, c, k, omega,flag);
					penalty[numVL/10-2][i] = res;
				}
				
				for(int i=0;i<10;i++) {
					double res = Tabu(priority, stop, n, numP, pSize, period, delayR, jitter,  R,  T, c, k, omega,0);
					penalty1[numVL/10-2][i] = res;
				}
				priorityRecord.add(priority);
				priorityRecord.remove(0);
			}	
			writeArrayToExcel(penalty,numP+"SFFP.txt");
			writeArrayToExcel(penalty1,numP+"SFNFP.txt");
		}
		
		
	}

}
