package AFDX;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;


public class Tabu1 {
	public static double NFP(List<Integer> hp, List<Integer> ep, List<Integer> lp, int[] pSize, double[] period, double[] jitter, int i, double R, double T) {/*state of the art without frame preemption*/
		double sum = 0;
		double mSize = 0;
		ServiceCurve sc = new ServiceCurve(R,T);
		for(int j = 0;j<ep.size();j++) {
			double rate = (double)(pSize[ep.get(j)])*8.0/period[ep.get(j)];
			double burst = (double)(pSize[ep.get(j)])*8;
			sc.reduceRate(rate);
			sc.increaseT(burst);
			sum = sum+rate;
		}
		for(int j = 0;j<hp.size();j++) {
			double rate = (double)(pSize[hp.get(j)])*8.0/period[hp.get(j)];
			double burst = (double)(pSize[hp.get(j)])*8;
			sc.reduceRate(rate);
			sc.increaseT(burst);
			sum = sum+rate;
		}
		for(int j=0;j<lp.size();j++) {
			if(pSize[lp.get(j)]>mSize) {
				mSize = pSize[lp.get(j)];
			}
		}
		sc.increaseT(mSize*8);		
	    double delay = sc.getDelay(pSize[i]*8);
		return delay;		
	}
	public static double FP(List<Integer> hp, List<Integer> ep, List<Integer> lp, int[] pSize, double[] period, double[] jitter, int i, double R, double T) {/*state of the art with frame preemption*/
		int m = pSize.length;
		ServiceCurve sc = new ServiceCurve(R,T);
		for(int j = 0;j<ep.size();j++) {
			double rate = (double)(pSize[ep.get(j)])*8.0/period[ep.get(j)];
			double burst = (double)(pSize[ep.get(j)])*8;
			sc.reduceRate(rate);
			sc.increaseT(burst);
		}
		for(int j = 0;j<hp.size();j++) {
			double rate = (double)(pSize[hp.get(j)])*8.0/period[hp.get(j)];
			double burst = (double)(pSize[hp.get(j)])*8;
			sc.reduceRate(rate);
			sc.increaseT(burst);
		}

	    double delay = sc.getDelay(pSize[i]*8);
		return delay;		
	}
	public static void findSets(int[] priority, List<Integer> hp, List<Integer> ep, List<Integer> lp, int i) {/*find the sets hp, ep and lp corresponding to flow i. hp contains flows having higher priorities than flow i.
	lp contains flows with lower priorities. ep contains flows with the same priority as flow i*/
		int m = priority.length;
		for(int j=0;j<m;j++) {
			if(j==i) continue;
			if(priority[j]<priority[i]) {
				lp.add(j);
			}else if (priority[j] == priority[i]) {
				ep.add(j);
			}else {
				hp.add(j);
			}
		}
	}
	public static void generateDiversity(int[] priority, int numP) {/*generate random array, which is totally different from array priority, to escape the local optima*/
		int m = priority.length;
		for(int i=0;i<m;i++) {
			Random rand = new Random();		
			int temp = rand.nextInt(numP);
			while(temp == priority[i]) {
				temp = rand.nextInt(numP);
			}
			priority[i] = temp;
		}
	}
	public static void generateNeighbors(List<int[]> neighbors, int n, int[] priority, int numP){/*generate neighbors, which only contain one different element from array priority. */
	    int m = priority.length;
	    Random rand = new Random();
	    
		while(neighbors.size()<n) {
			int temp = rand.nextInt(m);
			int[] tempP = priority.clone();
			tempP[temp] = rand.nextInt(numP);
			if(neighbors.contains(tempP)) continue;
			neighbors.add(tempP);
		}
		
	}
	public static double penalty(double[] delay, double[] delayR, double c) {/*calculate the penalty function*/
		double res = 0;
		for(int i=0;i<delay.length;i++) {
			if(delay[i]>delayR[i]) {
				res = res + c*(delay[i]-delayR[i]);
			}
		}
		return res;
	}
	
    /*using Tabu search to minimize the penalty function*/
	public static double Tabu(int[] priority, int stop, int n, int numP, int[] pSize, double[] period, double[] delayR, double[] jitter, double R, double T, double c, int k, int omega, int flag) {
		int[] current = priority.clone();
		int[] best = priority.clone();
		double bestP = getPenalty(best, pSize, period, delayR, jitter, R, T, c, flag);

		int iter = 0;
		List<int[]> tabu = new ArrayList<>();
	    List<Integer> tabuIter = new ArrayList<>();
	    int stagnation = 0;
		while(iter<stop) {
			iter++;
			if(tabu.contains(current)) {
				generateDiversity(current, numP);
			}
			tabu.add(current);
			tabuIter.add(k);
			List<int[]> neighbors = new ArrayList<>();
			generateNeighbors(neighbors, n, current, numP);
			
			for(int i=0;i<n;i++) {
				current = neighbors.get(i);
				if(tabu.contains(current)) continue;
				double p = getPenalty(current, pSize, period, delayR, jitter, R, T, c, flag);
				if(p<bestP) {
					best = current;
					bestP = p;
					stagnation = 0;
				}else {
					stagnation++;
				}
				tabu.add(current);
				tabuIter.add(k);
				if(stagnation == omega) {
					break;
				}else {
					for(int j=0;i<tabu.size();i++) {
						tabuIter.set(j, tabuIter.get(j)-1);
					}
					for(int j=0;i<tabu.size();i++) {
						if(tabuIter.get(j) <= 0) {
							tabu.remove(j);
							tabuIter.remove(j);
						}
						j--;
					}
				}
			}
			generateDiversity(current, numP);
		}
		//System.out.println("res of Tabu: "+Arrays.toString(best));
		priority = best;
		return bestP;
	}
	/*function to write results to a file*/
	public static void writeArrayToExcel(double[][] data, String string) {
        int rowNum = data.length;
        int columnNum = data[0].length;
        try {
            FileWriter fw = new FileWriter(string);
            for (int i = 0; i < rowNum; i++) {
                for (int j = 0; j < columnNum; j++)
                    fw.write(data[i][j]+ "\t"); 
                fw.write("\n"); 
            }
            fw.close();
        }
        catch (IOException e){
            e.printStackTrace();
        }
	}
	/*calculate the penalty function when the priority assignment is array priority*/
	public static double getPenalty(int[] priority, int[] pSize, double[] period, double[] delayR, double[] jitter, double R, double T, double c, int flag) {
		int m = priority.length;
		double[] delay = new double[m];
		for(int i=0;i<m;i++) {
			List<Integer> hp = new ArrayList<>();
			List<Integer> ep = new ArrayList<>();
			List<Integer> lp = new ArrayList<>();
			findSets(priority,hp,ep,lp,i);			
			//System.out.println(res);
			if(flag == 1) {
				delay[i] = FP(hp, ep, lp, pSize, period, jitter, i, R, T);
			}else {
				delay[i] = NFP(hp, ep, lp, pSize, period, jitter, i, R, T);
			}
			
			//System.out.println(delay[i]);
		}
		double res = penalty(delay, delayR, c);
		return res;		
	}
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		// TODO Auto-generated method stub
		int numVL = 20;
		List<int[]> priorityRecord2FP = new ArrayList<>();
		List<int[]> priorityRecord8FP = new ArrayList<>();
		List<int[]> priorityRecord2NFP = new ArrayList<>();
		List<int[]> priorityRecord8NFP = new ArrayList<>();
		double[][] penalty = new double[9][10];
		double[][] penalty1 = new double[9][10];
		for(int numP = 2;numP<=8;numP =numP+6) {
			for(numVL = 100;numVL>=20;numVL=numVL-10) {
				int[] priority = new int[numVL];/*priority of each VL*/
//				double[] BAG = {2, 4, 8, 16, 32, 64, 128};
//				int[] size = {32, 64, 128, 256, 512, 1024, 1500};
				double[] BAG = {2, 4, 8, 16}; /*BAG value candidates of VLs*/
				int[] size = {128, 256, 512, 1024};/*Maximum frame size candidates of VLs*/
				for(int i=0;i<numVL;i++) {
		        	priority[i] = i%numP;
		        }
				int[] previousPriority = new int[numVL+10];
				

		        
				double[] period = new double[numVL];/*BAG/period value of each VL*/
				double[] jitter =  new double[numVL];/*jitter of each VL*/
				for(int i=0;i<priority.length;i++) {
		        	period[i] = BAG[i%4]/1000;
		        	
				}
				int[] pSize = new int[numVL];
				for(int i=0;i<numVL;i++) {
					pSize[i] = size[i%4];
				}
				double[] delayR = new double[numVL];
				for(int i=0;i<numVL;i++) {
					delayR[i] = (double)pSize[i]/64/5000; /*Delay requirements all VLs*/
				}
				int stop = 10000;/*number of iterations of Tabu search*/
				double R = 500000*200; /*bandwidth of the switch*/
				double T = 0.001;/*delay of the switch*/
				double c = 100;/*constant to calculate penalty values*/
				int k = 10;
				int n = 10;
				int omega = 4;/*if there is no decrease of the penalty value in the current set of neighbors for omega times iterations, generate totally different */
				int flag = 1;/*1 means the switch has frame preemption. Otherwise not*/
				int numE = 10;
				
				for(int i=0;i<numE;i++) {
					
					double min = 10000;
					int index = 0;
					if(i == 0) {
						for(int j=0;j<numE;j++) {
							if(numVL/10-3<0) break;
							if(min>penalty[numVL/10-3][j]) {
								min = penalty[numVL/10-3][j];
								index = j;
							}
						}
						if(priorityRecord2FP.size() != 0 && numP == 2) {
							previousPriority = priorityRecord2FP.get(index);
							for(int j=0;j<numVL;j++) {
								priority[j] = previousPriority[j];
							}
							priorityRecord2FP.clear();
						}
						if(priorityRecord8FP.size() != 0 && numP == 8) {
							previousPriority = priorityRecord8FP.get(index);
							for(int j=0;j<numVL;j++) {
								priority[j] = previousPriority[j];
							}
							priorityRecord8FP.clear();
						}
					}
					//System.out.println(Arrays.toString(priority));
					
					double res = Tabu(priority, stop, n, numP, pSize, period, delayR, jitter,  R,  T, c, k, omega,flag);
					//System.out.println(numVL+" "+getPenalty(priority, pSize, period, delayR, jitter, R, T, c, flag));
					penalty[numVL/10-2][i] = res;
					if(numP == 2 ) {					
						priorityRecord2FP.add(priority);
					    //System.out.println(Arrays.toString(priorityRecord2FP.get(0)));
						
					}else {
						priorityRecord8FP.add(priority);
					}
					if(numP == 2) {
					}
					
				}			
				for(int i=0;i<numE;i++) {
					double min = 10000;
					int index = 0;
					if(i == 0) {
						for(int j=0;j<numE;j++) {
							if(numVL/10-3<0) break;
							if(min>penalty[numVL/10-3][j]) {
								min = penalty[numVL/10-3][j];
								index = j;
							}
						}
						if(priorityRecord2NFP.size() != 0 && numP == 2) {
							previousPriority = priorityRecord2NFP.get(index);
							for(int j=0;j<numVL;j++) {
								priority[j] = previousPriority[j];
							}
							priorityRecord2NFP.clear();
						}

						if(priorityRecord8NFP.size() != 0 && numP == 8) {
							previousPriority = priorityRecord8NFP.get(index);
							for(int j=0;j<numVL;j++) {
								priority[j] = previousPriority[j];
							}
							priorityRecord8NFP.clear();
						}
						
					}
					
					double res = Tabu(priority, stop, n, numP, pSize, period, delayR, jitter,  R,  T, c, k, omega,0);
					penalty1[numVL/10-2][i] = res;
					if(numP == 2 ) {					
						priorityRecord2NFP.add(priority);
											
					}else {
						priorityRecord8NFP.add(priority);
					}
				}
				
				
			}	
			writeArrayToExcel(penalty,numP+"FP.txt");
			writeArrayToExcel(penalty1,numP+"NFP.txt");
		}
	}

}
