//package AFDX;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

public class hybridAlgorithm {
	public static double penalty(double[] delay, double[] delayR, double c) {
		double res = 0;
		for(int i=0;i<delay.length;i++) {
			if(delay[i]>delayR[i]) {
				res = c*(delay[i]-delayR[i]);
			}
		}
		return res;
	}
	public static double getPenalty(int[] priority, int[] pSize, double[] period, double[] delayR, double[] jitter, double R, double T, double c) {
		int m = priority.length;
		double[] delay = new double[m];
		for(int i=0;i<m;i++) {
			List<Integer> hp = new ArrayList<>();
			List<Integer> ep = new ArrayList<>();
			List<Integer> lp = new ArrayList<>();
			findSets(priority,hp,ep,lp,i);			
			int res = FPNFP(hp, ep, lp, pSize, period, jitter, i, R);	
			//System.out.println(res);
			delay[i] = (R*T+(double)res*8+(double)pSize[i]*8)/R;
			//System.out.println(delay[i]);
		}
		double res = penalty(delay, delayR, c);
		return res;		
	}
	public static void findSets(int[] priority, List<Integer> hp, List<Integer> ep, List<Integer> lp, int i) {
		int m = priority.length;
		for(int j=0;j<m;j++) {
			if(j==i) continue;
			if(priority[j]<priority[i]) {
				lp.add(j);
			}else if (priority[j] == priority[i]) {
				ep.add(j);
			}else {
				hp.add(j);
			}
		}
	}
	public static int FPNFP(List<Integer> hp, List<Integer> ep, List<Integer> lp, int[] pSize, double[] period, double[] jitter, int i, double R) {
		int res = 0;
		boolean flag = true;
		List<Integer> l = new ArrayList<>();
		int k = hp.size();
		for(int j=0;j<k;j++) {
			l.add(1);
		}
		double J = jitter[i];
		double ipJ = 0;
		int mSize = 0;
		for(int j=0;j<k;j++) {
			res = res+pSize[hp.get(j)];
			J=Math.min(J,jitter[hp.get(j)]);
		}
		for(int j=0;j<lp.size();j++) {
			if(pSize[lp.get(j)] > mSize) {
				mSize = pSize[lp.get(j)];
				ipJ = jitter[lp.get(j)];
				
			}
		}
		res = res+mSize;
		for(int j=0;j<ep.size();j++) {
			res = res+pSize[ep.get(j)];
			J=Math.min(J,jitter[ep.get(j)]);
		}
		J = Math.max(J, ipJ);
		//System.out.println("res: "+i+": "+res);
		while(flag) {
			flag = false;
			int j = 0;
			
			while(j<k) {
				if((double)res*8.0/R+J > period[hp.get(j)]*(l.get(j))){
					flag = true;
					l.set(j,l.get(j)+1);
					res = res+pSize[hp.get(j)];
				}
				j = j+1;
			}
			
		}
		return res;		
	}
    public static int generateDistance(double deltaf, double alpha, int D, double epsilon1){
    	int res = 1;
    	double sigma = 0;
    	if(deltaf >= Math.exp(alpha)) {
    		sigma = 1-epsilon1;
    	}else {
    		sigma = alpha/Math.log(deltaf);
    	}
    	double r = Math.random();
    	double c = 0;
    	for(int i=1;i<=D;i++) {
    		c = c+(1-sigma)/(1-Math.pow(sigma, D))*Math.pow(sigma, i-1);
    		if(c<=r){
    			res = i;
    		}else {
    			break;
    		}
    	}
    	return res;
    }
    public static int[] generateNeighbor(int[] x, int d, int k) {
    	int i = 0;
    	int[] x1 = x.clone();
    	while(i<k) {
    		int m = (int)(x.length*Math.random());
    		if(x1[m]>d) {
    			for(int j=0;j<m;j++) {
    				if(x1[j] == x1[m]-d) {
    					int temp = x1[m];
    					x1[m] = x1[j];
    					x1[j] = temp;
    				}
    			}
    		}else {
    			for(int j=0;j<m;j++) {
    				if(x1[j] == x1[m]+d) {
    					int temp = x1[m];
    					x1[m] = x1[j];
    					x1[j] = temp;
    				}
    		    }
    			
    	   }
    		i = i+1;
    	}
    	return x1;    	    	
    }
    public static int[] rVNS(int[] x, double epsilon2, int kmax, double alpha, double epsilon1, int n, int numP, int[] pSize, double[] period, double[] delayR, double[] jitter, double R, double T, double c) throws Exception {
    	double deltap = 1;
    	double prevp = getPenalty(x, pSize, period, delayR,jitter, R, T, c);
    	double curp = 0;
    	
    	while(deltap>epsilon2) {   		
    		int k=1;
    		
	    	double prevp0 = prevp;
    		while(k<=kmax) {
    			//System.out.println(1);
    			int d = generateDistance(deltap,alpha,x.length/2,epsilon1);
    			int[] x1 = generateNeighbor(x, d, k);    		
    			double curp0 = getPenalty(x1, pSize, period, delayR,jitter, R, T, c);
    			
    			if(curp0<prevp0) {
    				x = x1.clone();
    				prevp0 = curp0;
    				
    			}
    			if(k == kmax) curp = prevp0;
    			k = k+1;    			
    		}
    		deltap = (prevp-curp)/prevp;
    		prevp = curp;
    		//System.out.print(deltaf);
    	}
    	return x;
    }
    public static int[] generateDiversity(int[] priority, int numP) {
		int m = priority.length;
		int[] res = new int[m];
		for(int i=0;i<m;i++) {
			Random rand = new Random();		
			int temp = rand.nextInt(numP);
			while(temp == priority[i]) {
				temp = rand.nextInt(numP);
			}
			res[i] = temp;
		}
		return res;
	}
    public static double GArVNS(int tmax, int popsize, double pmutation,double epsilon2, int kmax, double alpha, double epsilon1,int[] priority, int n, int numP, int[] pSize, double[] period, double[] delayR, double[] jitter, double R, double T, double c) throws Exception{
            
    	List<int[]> pop = new ArrayList<>();
    	pop.add(priority);
    	
        for(int i=0;i<popsize;i++) {
        	pop.add(generateDiversity(priority, numP));
        }
        
    	List<Double> penalties = new ArrayList<>();
    	double minP = Integer.MAX_VALUE;
    	double worstP = 0;
    	int index = 0;
    	int indexw = 0;
    	
    	for(int i=0;i<popsize;i++) {
    		double penalty = getPenalty(pop.get(i), pSize, period, delayR,jitter, R, T, c);
    		penalties.add(penalty);
    		if(penalty<minP) {
    			minP = penalty;
    			index = i;
    		}
    		if(penalty>worstP) {
    			worstP = penalty;
    			indexw = i;
    		}
    	}
    	
    	int t1 = 0;
        while(t1<tmax) {
        	t1++;
        	
        	int[] p1 = pop.get(index);	
        	int temp = (int)Math.random()*(popsize);
        	while(temp == index) {
        		temp = (int)(Math.random()*(popsize));
        	}
        	int[] p2 = pop.get(temp);
        	int[] child = new int[p1.length];
        	for(int i=0;i<p1.length;i++) {
        		if(Math.random()<0.5) {
        			child[i] = p1[i];
        		}else {
        			child[i] = p2[i];
        		}
        	}
        	if(Math.random()<pmutation) {
        		int index1 = (int)(Math.random()*child.length);
        		int index2 = (int)(Math.random()*child.length);
        		int temp1 = child[index1];
        		child[index1] = child[index2];
        		child[index2] = temp1;
        	}
        	child = rVNS(child, epsilon2, kmax, alpha, epsilon1, n, numP, pSize,  period,  delayR, jitter, R,  T, c);
            
        	double penaltyC = getPenalty(child, pSize, period, delayR,jitter, R, T, c);
        	if(penaltyC>worstP) {
        		continue;
        	}
        	if(penaltyC<minP) {
        		minP = penaltyC;
        		index = popsize-1;
        		penalties.remove(indexw);
        		pop.remove(indexw);
        	}
        	if(penaltyC>minP) {
        		if(indexw<index) {
        			index = index-1;
        		}
        		penalties.remove(indexw);
        		pop.remove(indexw);
        	}
        	penalties.add(penaltyC);
        	pop.add(child);
        }
       // System.out.println(Arrays.toString(pop.get(index)));
    	return penalties.get(index);
    	
    }
    public static int[] shuffle(int[] x) {
		Random rand = new Random();
		int[] res = new int[x.length];
		res = x.clone();
		for (int i = 0; i < res.length; i++) {
			int randomIndexToSwap = rand.nextInt(res.length);
			int temp = res[randomIndexToSwap];
			res[randomIndexToSwap] = res[i];
			res[i] = temp;
		}
		
		return res;
    }
	public static void main(String[] args) throws Exception {
		int numVL = 200;
		int[] priority = new int[numVL];
		double[] BAG = {2, 4, 8, 16, 32, 64, 128};
		int[] size = {32, 64, 128, 256, 512, 1024, 1500};
		int numP = 8;
        for(int i=0;i<priority.length;i++) {
        	Random rand = new Random();
        	priority[i] = rand.nextInt(numP);
        }
		double[] period = new double[numVL];
		double[] jitter =  new double[numVL];
		for(int i=0;i<priority.length;i++) {
        	period[i] = BAG[i%7]/1000;
		}
		int[] pSize = new int[numVL];
		for(int i=0;i<numVL;i++) {
			pSize[i] = size[i%7];
		}
		double[] delayR = new double[numVL];
		for(int i=0;i<numVL;i++) {
			delayR[i] = pSize[i]/64/250;
		}
		double R = 100000000;
		double T = 0.001;
		double c = 100;
		int n = 10;
		double epsilon2 = 0.001;
		double epsilon1 = 0.001;
		double alpha = -0.5;
		int kmax = 5;
		int tmax = 1000;
		int popsize = 10;
		double pmutation = 0.5;
		long startTime=System.currentTimeMillis(); 
		for(int i=0;i<10;i++) {
			double res = GArVNS(tmax, popsize, pmutation, epsilon2, kmax, alpha, epsilon1, priority, n, numP, pSize, period, delayR, jitter, R, T, c);
		    System.out.println(res);
		}
		long endTime=System.currentTimeMillis(); //ȡʱ  
		System.out.println("ʱ䣺 "+(endTime-startTime)+"ms");

	}

}
