
package implementation;
import java.util.LinkedList;
import java.util.List;

import org.networkcalculus.dnc.curves.ArrivalCurve;
import org.networkcalculus.dnc.curves.Curve;
import org.networkcalculus.dnc.curves.ServiceCurve;
import org.networkcalculus.dnc.network.server_graph.Flow;
import org.networkcalculus.dnc.network.server_graph.Server;
import org.networkcalculus.dnc.network.server_graph.ServerGraph;
import org.networkcalculus.dnc.network.server_graph.ServerGraphFactory;
import org.networkcalculus.dnc.network.server_graph.Turn;

public class FourTandem_ServerGraph implements ServerGraphFactory{
	private final int sc_R = 10;
	private final double sc_T =  0.0016;
	private final double ac_rfol = 2.8;
	private final double ac_bfol = 0.0014;
	private final double ac_r = 5.2;
	private final double ac_b = 0.0026;
	private Server s0, s1, s2, s3;
	private Turn t_s0_s1, t_s1_s2, t_s2_s3, t_s1_s0;
	
	private ServiceCurve service_curve = Curve.getFactory().createRateLatency(sc_R, sc_T);
	private ArrivalCurve arrival_curvefol = Curve.getFactory().createTokenBucket(ac_rfol, ac_bfol);
	private ArrivalCurve arrival_curve = Curve.getFactory().createTokenBucket(ac_r, ac_b);
	private ServerGraph server_graph;

	public FourTandem_ServerGraph() {
		server_graph = createServerGraph();
	}

	public ServerGraph getServerGraph() {
		return server_graph;
	}

	public ServerGraph createServerGraph() {
		server_graph = new ServerGraph();

		s0 = server_graph.addServer(service_curve);
		s1 = server_graph.addServer(service_curve);
		s2 = server_graph.addServer(service_curve);
		s3 = server_graph.addServer(service_curve);

		try {
			t_s0_s1 = server_graph.addTurn(s0, s1);
			t_s1_s2 = server_graph.addTurn(s1, s2);
			t_s2_s3 = server_graph.addTurn(s2, s3);


		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

		List<Turn> f5_path = new LinkedList<Turn>();
		f5_path.add(t_s0_s1);
		f5_path.add(t_s1_s2);
		f5_path.add(t_s2_s3);
		List<Turn> f2_path = new LinkedList<Turn>();
		f2_path.add(t_s2_s3);


		try {
			server_graph.addFlow("f1", arrival_curvefol, f5_path);
			server_graph.addFlow("f2", arrival_curve, f2_path);
			//server_graph.addFlow("f2", arrival_curve, f2_path);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

		return server_graph;
	}

	public void reinitializeCurves() {
		service_curve = Curve.getFactory().createRateLatency(sc_R, sc_T);
		for (Server server : server_graph.getServers()) {
			server.setServiceCurve(service_curve);
		}

		arrival_curve = Curve.getFactory().createTokenBucket(ac_r, ac_b);
		for (Flow flow : server_graph.getFlows()) {
			flow.setArrivalCurve(arrival_curve);
		}
	}

}
