package implementation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.networkcalculus.dnc.AnalysisConfig;
import org.networkcalculus.dnc.AnalysisConfig.ArrivalBoundMethod;
import org.networkcalculus.dnc.AnalysisConfig.MaxScEnforcement;
import org.networkcalculus.dnc.AnalysisConfig.MultiplexingEnforcement;
import org.networkcalculus.dnc.network.server_graph.Flow;
import org.networkcalculus.dnc.network.server_graph.ServerGraph;
import org.networkcalculus.dnc.numbers.Num;
import org.networkcalculus.dnc.tandem.analyses.PmooAnalysis;
import org.networkcalculus.dnc.tandem.analyses.SeparateFlowAnalysis;
import org.networkcalculus.dnc.tandem.analyses.TotalFlowAnalysis;
public class hybridAlgorithm {
    public static double fitness(double[][] d, double[][] t, double beta) {
    	double res = 0;
    	int m = d.length;
    	for(int i=0;i<m;i++) {
    		for(int j=0;j<m;j++) {
    			if(i == j) continue;
    			if(d[i][j]/t[i][j] <=1 ) {
        			res = res+d[i][j]/t[i][j];
        		}else {
        			res = res+beta*d[i][j]/t[i][j];
        		}    	
    		}
    				
    	}
    	return res;
    }
    public static int generateDistance(double deltaf, double alpha, int D, double epsilon1){
    	int res = 1;
    	double sigma = 0;
    	if(deltaf >= Math.exp(alpha)) {
    		sigma = 1-epsilon1;
    	}else {
    		sigma = alpha/Math.log(deltaf);
    	}
    	double r = Math.random();
    	double c = 0;
    	for(int i=1;i<=D;i++) {
    		c = c+(1-sigma)/(1-Math.pow(sigma, D))*Math.pow(sigma, i-1);
    		if(c<=r){
    			res = i;
    		}else {
    			break;
    		}
    	}
    	return res;
    }
    public static int[] generateNeighbor(int[] x, int d, int k) {
    	int i = 0;
    	int[] x1 = x.clone();
    	while(i<k) {
    		int m = (int)(x.length*Math.random());
    		if(x1[m]>d) {
    			for(int j=0;j<m;j++) {
    				if(x1[j] == x1[m]-d) {
    					int temp = x1[m];
    					x1[m] = x1[j];
    					x1[j] = temp;
    				}
    			}
    		}else {
    			for(int j=0;j<m;j++) {
    				if(x1[j] == x1[m]+d) {
    					int temp = x1[m];
    					x1[m] = x1[j];
    					x1[j] = temp;
    				}
    		    }
    			
    	   }
    		i = i+1;
    	}
    	return x1;    	    	
    }
    public static double[][] getDelay(double[][] rate, double[][] burst, int[] x, int nSwitch) throws Exception{
    	int m = x.length;
    	double[][] res = new double[m][m];
    	for(int i=0;i<m;i++) {
    		for(int j=0;j<m;j++) {
    			if(i==j) continue;
    			hybrid_ServerGraph network_factory = new hybrid_ServerGraph(rate,burst, x, nSwitch, i, j);
    			ServerGraph network = network_factory.getServerGraph();
    			Set<Flow> s = new HashSet<>();
    			s = network.getFlows();
    			List<Flow> l = new ArrayList<>(s);
    			Flow fol = null;
    			for(int k=0;k<l.size();k++) {
    				if(l.get(k).getAlias() == "fol") {
    					fol = l.get(k);
    				}
    			}
    			Set<ArrivalBoundMethod> arrival_bound_methods = new HashSet<ArrivalBoundMethod>(Collections.singleton(ArrivalBoundMethod.AGGR_PBOO_CONCATENATION));
    			AnalysisConfig configuration = new AnalysisConfig(MultiplexingEnforcement.GLOBAL_FIFO,MaxScEnforcement.SERVER_LOCAL,MaxScEnforcement.SERVER_LOCAL,arrival_bound_methods, true,false);	
    			SeparateFlowAnalysis sfa = new SeparateFlowAnalysis(network, configuration);
    			sfa.performAnalysis(fol);
    			Num res2 = sfa.getDelayBound();
    			res[i][j] = res2.doubleValue();
    			
    		}
    	}
    	//System.out.println(res[0][1]+"res[i][j]");
    	return res;
    	
    }
    public static int[] rVNS(int[] x, double epsilon2, int kmax, double alpha, double epsilon1, double[][] rate, double[][] burst, int nSwitch, double[][] t, double beta) throws Exception {
    	double deltaf = 1;
    	double[][] delay = getDelay(rate, burst, x, nSwitch);
    	double prevf = fitness(delay,t,beta);
    	double curf = 0;
    	
    	while(deltaf>epsilon2) {   		
    		int k=1;
    		
	    	double prevf0 = prevf;
    		while(k<=kmax) {
    			//System.out.println(1);
    			int d = generateDistance(deltaf,alpha,nSwitch/2,epsilon1);
    			System.out.println(d);
    			int[] x1 = generateNeighbor(x, d, k);    
    			double[][] delay1 = getDelay(rate, burst, x1, nSwitch);			
    			double curf0 = fitness(delay1,t,beta);
    			
    			if(curf0<prevf0) {
    				x = x1.clone();
    				prevf0 = curf0;
    				
    			}
    			if(k == kmax) curf = prevf0;
    			k = k+1;    			
    		}
    		System.out.println(curf+" curf");
    		deltaf = (prevf-curf)/prevf;
    		prevf = curf;
    		//System.out.print(deltaf);
    	}
    	return x;
    }
    public static int[] GArVNS(int tmax,int nDevices,int psize, double pmutation,double epsilon2, int kmax, double alpha, double epsilon1, double[][] rate, double[][] burst, int nSwitch, double[][] t, double beta) throws Exception{
    	int m = nDevices/nSwitch;
    	int[] x = new int[nDevices];
    	for(int i=0; i<nDevices;i=i+m) {
    		
    		for(int j=i;j<i+m && j<nDevices;j++) {
    			x[j] =  Math.min(nSwitch-1, i/m);
    			System.out.println(x[j]);
    			
    		}
    	}
    	List<int[]> p = new ArrayList<>();
    	
    	for(int i=0;i<psize;i++) {
    		int[] res = shuffle(x);
    		p.add(res);
    	}
    	double[][] fitness = new double[2][psize];
    	for(int i=0;i<psize;i++) {
    		double[][] delay = getDelay(rate, burst, p.get(i), nSwitch);
    		fitness[0][i] = fitness(delay,t,beta);
    		fitness[1][i] = i;
    	}
    	Comparator<double[]> cmp = new Comparator<double[]>() {

			@Override
			public int compare(double[] o1, double[] o2) {
				if(o1[0]-o2[0]>0) {
					return 1;
				}
				else return -1;
			}
    		
    	};
    	List<double[]> l = new ArrayList<>();
    	for(int i=0;i<3;i++) {
    		double[] temp = new double[2];
    		temp[0] = fitness[0][i];
    		temp[1] = fitness[1][i];
    		l.add(temp);
    	}
    	Collections.sort(l,cmp);
    	int t1 = 0;
        while(t1<tmax) {
        	int[] p1 = p.get((int)l.get(0)[1]);	
        	int[] p2 = p.get((int)(Math.random()*(psize)));
        	int[] c = new int[p1.length];
        	for(int i=0;i<p1.length;i++) {
        		if(Math.random()<0.5) {
        			c[i] = p1[i];
        		}else {
        			c[i] = p2[i];
        		}
        	}
        	if(Math.random()<0.15) {
        		int index1 = (int)(Math.random()*c.length);
        		int index2 = (int)(Math.random()*c.length);
        		int temp = c[index1];
        		c[index1] = c[index2];
        		c[index2] = temp;
        	}
        	c = rVNS(c, epsilon2, kmax, alpha, epsilon1, rate, burst, nSwitch,t, beta);
        	double[][] delayC = getDelay(rate, burst, c, nSwitch);
        	double fitnessC = fitness(delayC,t,beta);
        	if(l.get(l.size()-1)[0]>fitnessC) {
        		int target = (int) l.get(l.size()-1)[1];
        		p.set(target, c);
        		double[] r = new double[2];
        		r[0] = fitnessC;
        		r[1] = target;
        		l.set(l.size()-1, r);
        		Collections.sort(l,cmp);  		
        	}
        	t1++;
        }
    	return p.get((int) l.get(0)[1]);
    	
    }
    public static int[] shuffle(int[] x) {
		Random rand = new Random();
		int[] res = new int[x.length];
		res = x.clone();
		for (int i = 0; i < res.length; i++) {
			int randomIndexToSwap = rand.nextInt(res.length);
			int temp = res[randomIndexToSwap];
			res[randomIndexToSwap] = res[i];
			res[i] = temp;
		}
		return res;
    }
	public static void main(String[] args) throws Exception {
		// TODO Auto-generated method stub
//    	double[][] x = {{5,1,2},{0,1,2}};
//    	Comparator<double[]> cmp = new Comparator<double[]>() {
//
//			@Override
//			public int compare(double[] o1, double[] o2) {
//				if(o1[0]-o2[0]>0) {
//					return 1;
//				}
//				else return -1;
//			}
//    		
//    	};
//    	List<double[]> l = new ArrayList<>();
//    	for(int i=0;i<3;i++) {
//    		double[] temp = new double[2];
//    		temp[0] = x[0][i];
//    		temp[1] = x[1][i];
//    		l.add(temp);
//    	}
//    	Collections.sort(l,cmp);
//    	for(int i=0;i<3;i++) {
//    	    double[] temp = l.get(i);
//    		System.out.println(Arrays.toString(temp));
//    	}
    	
    	
		
		int[] x = {0,0,1,1,2,2,3,3,4,4,5,5};
		int m = x.length;
		
		double[][] rate = new double[x.length][x.length];
		double[][] burst = new double[x.length][x.length];
		double[][] t = new double[x.length][x.length];
		
		for(int i=0;i<m;i++) {
			for(int j=0;j<m;j++) {
				if(i!=j) {
					rate[i][j] = Math.random()*3;
					burst[i][j] = 0.1;
					t[i][j] = 0.3;
				}
			}
		}
		int nSwitch = 6;
		double epsilon2 = 0.001;
		double epsilon1 = 0.001;
		double beta = 100;
		double alpha = -0.5;
		int kmax = 5;
		int tmax = 10;
		int nDevices = m;
		int psize = 20;
		double pmutation = 0.15;
		double[][] delay0 = getDelay(rate, burst, x, nSwitch);
    	double f0 = fitness(delay0,t,beta);
    	System.out.println(f0);
		int[] res = GArVNS(tmax,nDevices, psize,  pmutation, epsilon2,  kmax,  alpha,  epsilon1, rate,  burst, nSwitch, t,  beta);
		double[][] delay = getDelay(rate, burst, res, nSwitch);
    	double f = fitness(delay,t,beta);
       
        System.out.println(f);

//    	for(int i=0;i<10;i++) {
//    		System.out.println(x[i]);
//    	}
//		double[][] res = getDelay(rate, burst, x, nSwitch);
//		for (int i=0;i<5;i++) {
//			for(int j=0;j<5;j++) {
//				System.out.println(res[i][j]);
//			}
//		}

	}

}
