
package implementation;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.networkcalculus.dnc.curves.ArrivalCurve;
import org.networkcalculus.dnc.curves.Curve;
import org.networkcalculus.dnc.curves.ServiceCurve;
import org.networkcalculus.dnc.network.server_graph.Flow;
import org.networkcalculus.dnc.network.server_graph.Server;
import org.networkcalculus.dnc.network.server_graph.ServerGraph;
import org.networkcalculus.dnc.network.server_graph.ServerGraphFactory;
import org.networkcalculus.dnc.network.server_graph.Turn;

public class hybrid_ServerGraph implements ServerGraphFactory{
	private final double sc_R = 100;
	private final double sc_T =  0.1;
	private final double ac_r = 0.67;
	private final double ac_b = 1;
	
	private Server[] sl;
	private Turn[] tl;
	private ArrivalCurve[][] aC;
	private ServiceCurve service_curve = Curve.getFactory().createRateLatency(sc_R, sc_T);
	private ArrivalCurve arrival_curve = Curve.getFactory().createTokenBucket(ac_r, ac_b);
	private ServerGraph server_graph;

	public hybrid_ServerGraph(double[][] rate, double[][] burst, int[] x, int nSwitch, int r, int c) {
		int m = x.length;
		aC = new ArrivalCurve[m][m];
	    for (int i=0;i<m;i++) {
	    	for(int j=0;j<m;j++) {
	    		if(rate[i][j] == 0) continue;
	    		aC[i][j] = Curve.getFactory().createTokenBucket(rate[i][j], burst[i][j]);
	    	}
	    }
	    sl = new Server[nSwitch];
	    tl = new Turn[nSwitch-1];
	    
	    server_graph = createServerGraph(x, sl, tl,r,  c);
	}

	public ServerGraph getServerGraph() {
		return server_graph;
	}
    public void swap(int src, int des) {
    	if(src>des) {
			int temp = src;
			src = des;
			des = temp;
		}
    }
	public ServerGraph createServerGraph(int[] x, Server[] sl, Turn[] tl, int r, int c) {
		server_graph = new ServerGraph();
		
		for(int i=0;i<sl.length;i++) {
			sl[i]  =  server_graph.addServer(service_curve);
		}
		
		try {
			for(int i=0;i<tl.length;i++) {
				tl[i] = server_graph.addTurn(sl[i],sl[i+1]);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		
		add_flow(r,c,x,server_graph,aC,"fol");
		
		
		for(int i=0;i<x.length;i++) {
			for(int j=0;j<x.length && i!=j;j++) {
				int src = x[r];
				int des = x[c];
				
				if(x[j] == des) {
					String alias = "f_"+i+"_"+j;
					add_flow(i,j,x,server_graph,aC,alias);
					continue;
				}
				if(x[i] == src) {
					String alias = "f_"+i+"_"+j;
					add_flow(i,j,x,server_graph,aC,alias);
					continue;
				}
				if(src == des || x[i] == x[j]) continue;
				if(src<des && x[i]<x[j]) {
					if((src>x[j] && src<=x[i]) || (des<=x[j]) && des>x[i]) {
						String alias = "f_"+i+"_"+j;
						add_flow(i,j,x,server_graph,aC,alias);
					}
					continue;
				}
				if(src>des && x[i]>x[i]) {
					if((src>x[j] && src<=x[i]) || (des<=x[j]) && des>x[i]) {
						String alias = "f_"+i+"_"+j;
						add_flow(i,j,x,server_graph,aC,alias);
					}
					continue;
				}
			}
		}


		return server_graph;
	}
    public void add_flow(int r, int c, int[] x, ServerGraph server_graph, ArrivalCurve[][] aC, String alias ) {
    	List<Turn> fol = new LinkedList<>();
		int src = x[r];
		int des = x[c];
		if(src == des) {
			try {
				
				server_graph.addFlow(alias,aC[r][c],sl[src]);
				
			}catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}else {
			
			if(src>des) {
				int temp = src;
				src = des;
				des = temp;
			}
			for(int i=src;i<des;i++) fol.add(tl[i]);
			try {
				server_graph.addFlow(alias,aC[r][c],fol);
			}catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}				
		}
    }
	public void reinitializeCurves() {
		service_curve = Curve.getFactory().createRateLatency(sc_R, sc_T);
		for (Server server : server_graph.getServers()) {
			server.setServiceCurve(service_curve);
		}

		arrival_curve = Curve.getFactory().createTokenBucket(ac_r, ac_b);
		for (Flow flow : server_graph.getFlows()) {
			flow.setArrivalCurve(arrival_curve);
		}
	}

	@Override
	public ServerGraph createServerGraph() {
		// TODO Auto-generated method stub
		return null;
	}

}
